@@ -5,10 +5,16 @@ import os
 from .asset_attributes import AssetAttributes
 from .assets import build_asset
 from .cache import SimpleCache
-from .compilers import (
-    CoffeeScriptCompiler, HandlebarsCompiler, LessCompiler, StylusCompiler)
 from .exceptions import FileNotFound
 from .processors import DirectivesProcessor
+from .utils import get_condition_func
+
+
+DEFAULT_PUBLIC_ASSETS = (
+    lambda path: not any(path.endswith(ext) for ext in ('.css', '.js')),
+    r'^css/style\.css$',
+    r'^js/script\.js$',
+)
 
 
 class Finders(list):
@@ -63,18 +69,6 @@ class Compilers(dict):
     compiler.
     """
 
-    def register_defaults(self):
-        """Register :class:`~gears.compilers.CoffeeScriptCompiler`,
-        :class:`~gears.compilers.HandlebarsCompiler`,
-        :class:`~gears.compilers.LessCompiler` and
-        :class:`~gears.compilers.StylusCompiler` for ``.coffee``,
-        ``.handlebars``, ``.less`` and ``.styl`` extensions, respectively.
-        """
-        self.register('.coffee', CoffeeScriptCompiler.as_handler())
-        self.register('.handlebars', HandlebarsCompiler.as_handler())
-        self.register('.less', LessCompiler.as_handler())
-        self.register('.styl', StylusCompiler.as_handler())
-
     def register(self, extension, compiler):
         """Register passed `compiler` with passed `extension`."""
         self[extension] = compiler
@@ -151,29 +145,6 @@ class Compressors(dict):
             del self[mimetype]
 
 
-class PublicAssets(list):
-    """The registry for public assets. It acts like a list of logical paths of
-    assets.
-    """
-
-    def register_defaults(self):
-        """Register ``css/style.css`` and ``js/script.js`` as public assets."""
-        self.register('css/style.css')
-        self.register('js/script.js')
-
-    def register(self, path):
-        """Register passed `path` as public asset."""
-        if path not in self:
-            self.append(path)
-
-    def unregister(self, path):
-        """Remove passed `path` from registry. If `path` does not found in the
-        registry, nothing happens.
-        """
-        if path in self:
-            self.remove(path)
-
-
 class Suffixes(list):
     """The registry for asset suffixes. It acts like a list of dictionaries.
     Every dictionary has three keys: ``extensions``, ``result_mimetype`` and
@@ -233,16 +204,18 @@ class Environment(object):
     """This is the central object, that links all Gears parts. It is passed the
     absolute path to the directory where public assets will be saved.
     Environment contains registries for file finders, compilers, compressors,
-    processors, supported MIME types and public assets.
+    processors and supported MIME types.
 
     :param root: the absolute path to the directory where handled public assets
                  will be saved by :meth:`save` method.
+    :param public_assets: a list of public assets paths.
     :param cache: a cache object. It is used by assets and dependencies to
                   store compilation results.
     """
 
-    def __init__(self, root, cache=None):
+    def __init__(self, root, public_assets=DEFAULT_PUBLIC_ASSETS, cache=None):
         self.root = root
+        self.public_assets = [get_condition_func(c) for c in public_assets]
         self.cache = cache if cache is not None else SimpleCache()
 
         #: The registry for file finders. See
@@ -261,11 +234,6 @@ class Environment(object):
         #: :class:`~gears.environment.Compressors` for more information.
         self.compressors = Compressors()
 
-        #: The registry for public assets. Only assets from this registry will
-        #: be saved to the :attr:`root` path. See
-        #: :class:`~gears.environment.PublicAsets` for more information.
-        self.public_assets = PublicAssets()
-
         #: The registry for asset preprocessors. See
         #: :class:`~gears.environment.Preprocessors` for more information.
         self.preprocessors = Preprocessors()
@@ -290,12 +258,8 @@ class Environment(object):
         return self._suffixes
 
     def register_defaults(self):
-        """Register default compilers, preprocessors, MIME types and public
-        assets.
-        """
-        self.compilers.register_defaults()
+        """Register default compilers, preprocessors and MIME types."""
         self.mimetypes.register_defaults()
-        self.public_assets.register_defaults()
         self.preprocessors.register_defaults()
 
     def find(self, item, logical=False):
@@ -326,8 +290,11 @@ class Environment(object):
                     continue
         elif logical:
             asset_attributes = AssetAttributes(self, item)
+            suffixes = self.suffixes.find(asset_attributes.mimetype)
+            if not suffixes:
+                return self.find(item)
             path = asset_attributes.path_without_suffix
-            for suffix in self.suffixes.find(asset_attributes.mimetype):
+            for suffix in suffixes:
                 try:
                     return self.find(path + suffix)
                 except FileNotFound:
@@ -341,13 +308,14 @@ class Environment(object):
                 return AssetAttributes(self, item), absolute_path
         raise FileNotFound(item)
 
-    def list(self, path, mimetype=None):
+    def list(self, path, mimetype=None, recursive=False):
         """Yield two-tuples for all files found in the directory given by
         ``path`` parameter. Result can be filtered by the second parameter,
         ``mimetype``, that must be a MIME type of assets compiled source code.
-        Each tuple has :class:`~gears.asset_attributes.AssetAttributes`
-        instance for found file path as first item, and absolute path to this
-        file as second item.
+        If ``recursive`` is ``True``, then ``path`` will be scanned
+        recursively. Each tuple has
+        :class:`~gears.asset_attributes.AssetAttributes` instance for found
+        file path as first item, and absolute path to this file as second item.
 
         Usage example::
 
@@ -360,7 +328,7 @@ class Environment(object):
         """
         found = set()
         for finder in self.finders:
-            for logical_path, absolute_path in finder.list(path):
+            for logical_path, absolute_path in finder.list(path, recursive=recursive):
                 asset_attributes = AssetAttributes(self, logical_path)
                 if mimetype is not None and asset_attributes.mimetype != mimetype:
                     continue
@@ -370,11 +338,11 @@ class Environment(object):
 
     def save(self):
         """Save handled public assets to :attr:`root` directory."""
-        for path in self.public_assets:
-            try:
-                self.save_file(path, str(build_asset(self, path)))
-            except FileNotFound:
-                pass
+        for asset_attributes, absolute_path in self.list('.', recursive=True):
+            logical_path = os.path.normpath(asset_attributes.logical_path)
+            if self.is_public(logical_path):
+                asset = build_asset(self, logical_path)
+                self.save_file(logical_path, str(asset))
 
     def save_file(self, path, source):
         filename = os.path.join(self.root, path)
@@ -385,3 +353,6 @@ class Environment(object):
             raise OSError("%s exists and is not a directory." % path)
         with open(filename, 'w') as f:
             f.write(source)
+
+    def is_public(self, logical_path):
+        return any(condition(logical_path) for condition in self.public_assets)
