@@ -1,4 +1,5 @@
 import os
+import re
 
 
 missing = object()
@@ -39,3 +40,20 @@ def unique(iterable, key=lambda x: x):
         if keyitem not in yielded:
             yielded.add(keyitem)
             yield item
+
+
+def listdir(path, recursive=False):
+    for dirpath, dirnames, filenames in os.walk(path):
+        if not recursive:
+            dirnames[:] = []
+        dirpath = os.path.relpath(dirpath, path)
+        for filename in filenames:
+            yield os.path.normpath(os.path.join(dirpath, filename))
+
+
+def get_condition_func(condition):
+    if callable(condition):
+        return condition
+    if isinstance(condition, basestring):
+        condition = re.compile(condition)
+    return lambda path: condition.search(path)
